import { Show, For, Switch, Match } from "solid-js";
import type { MenuItem } from "~/types/common";
import type { MenuName } from "~/contexts/MenuMobileContext";
import { useMenuMobileContext } from "~/contexts/MenuMobileContext";

type MenuProps = {
  class?: string;
  items: MenuItem[];
  maxLevel?: number;
  name: MenuName;
};

export default function Menu(props: MenuProps) {
  const [, { pushStackLevelWithLabel }] = useMenuMobileContext();
  const maxLevel = () => props.maxLevel || -1;

  return (
    <ul class={props.class}>
      <For each={props.items}>
        {(item) => (
          <>
            <li
              class={item.sanitized}
              onClick={(e) => {
                pushStackLevelWithLabel(props.name, item.name, e);
              }}
            >
              <MenuItemRender
                item={item}
                currentLevel={1}
                maxLevel={maxLevel()}
              />
            </li>
          </>
        )}
      </For>
    </ul>
  );
}

type MenuItemProps = {
  item: MenuItem;
  currentLevel: number;
  maxLevel?: number;
};

function MenuItemRender(props: MenuItemProps) {
  const maxLevel = () => props.maxLevel || -1;
  return (
    <>
      <Show when={props.item.href} fallback={<span>{props.item.name}</span>}>
        <Show
          fallback={<a href={props.item.href}>{props.item.name}</a>}
          when={
            props.item.name === "Espace presse" ||
            props.item.name === "Aménagement"
          }
        >
          <a href={props.item.href} target="_blank">
            {props.item.name}
          </a>
        </Show>
      </Show>
      <Show
        when={
          props.item.children &&
          (props.currentLevel < maxLevel() || maxLevel() === -1)
        }
      >
        <div class="flex-content">
          <ul>
            <For each={props.item.children}>
              {(child) => (
                <>
                  <li class={child.sanitized}>
                    <MenuItemRender
                      item={child}
                      currentLevel={props.currentLevel + 1}
                      maxLevel={maxLevel()}
                    />
                  </li>
                </>
              )}
            </For>
          </ul>
        </div>
      </Show>
    </>
  );
}
