import type { MenuItem, Settings } from "~/types/common";
import type { ParentProps } from "solid-js";
import { createEffect } from "solid-js";
import { onMount } from "solid-js";

import { MenuMobileContextProvider } from "~/contexts/MenuMobileContext";
import TopBar from "~/components/shared/TopBar";
import { Suspense } from "solid-js";
import Footer from "~/components/shared/Footer";
import {
  programBarShouldStack,
  topBarShouldStack,
  useStackingContext,
} from "~/contexts/StackingContext";
import { useWindowSize } from "@solid-primitives/resize-observer";
import { useViewportSizeContext } from "~/contexts/ViewportSizeContext";

type PageContentProps = {
  menuMain: MenuItem[];
  menuFooter: MenuItem[];
  settings: Settings;
};

export default function PageContent(props: ParentProps<PageContentProps>) {
  const [, { setTopBarHeaderStack, setProgramHeaderStack }] =
    useStackingContext();

  if (
    setTopBarHeaderStack === undefined ||
    setProgramHeaderStack === undefined
  ) {
    throw new Error("useStackingContext must be used within its provider");
  }

  const handleScroll = () => {
    setTopBarHeaderStack(topBarShouldStack());
    setProgramHeaderStack(programBarShouldStack());
  };

  onMount(() => (onscroll = handleScroll));

  const size = useWindowSize();

  const [
    viewPortState,
    {
      setViewPortIsLessThan768,
      setViewPortIsMoreThan768,
      setViewPortIsLessThan1024,
      setViewPortIsLessThan1210,
    },
  ] = useViewportSizeContext();

  createEffect(() => {
    setViewPortIsLessThan768!(size.width < 768);
    setViewPortIsMoreThan768!(size.width > 768);
    setViewPortIsLessThan1024!(size.width < 1024);
    setViewPortIsLessThan1210!(size.width < 1210);
  });

  return (
    <>
      <Suspense>
        <MenuMobileContextProvider>
          {JSON.stringify(viewPortState)}
          <TopBar menuMainItems={props.menuMain} settings={props.settings} />
        </MenuMobileContextProvider>
        <Suspense fallback={<p>Content loading</p>}>{props.children}</Suspense>
        <Footer menuItems={props.menuFooter} />
      </Suspense>
    </>
  );
}
