import type { ParentProps } from "solid-js";
import { RealEstateContext } from "~/types/common";

import { StackingContextProvider } from "~/contexts/StackingContext";
import { RequestContextProvider } from "~/contexts/RequestContext";
import { ViewportSizeContextProvider } from "~/contexts/ViewportSizeContext";
import { DrupalSettingsProvider } from "~/contexts/DrupalSettingsContext";

import { createResource } from "solid-js";
import { getMenuFooter, getMenuMain, getSettings } from "~/lib/fetcher";

import PageContent from "~/components/shared/PageContent";

export default function RootContexts(props: ParentProps) {
  const [menuMain] = createResource(() => getMenuMain(), { deferStream: true });
  const [menuFooter] = createResource(() => getMenuFooter(), {
    deferStream: true,
  });
  const [settings] = createResource(() => getSettings(), { deferStream: true });

  return (
    <>
      <DrupalSettingsProvider settings={settings()!}>
        <ViewportSizeContextProvider
          viewPortIsLessThan768={false}
          viewPortIsMoreThan768={false}
          viewPortIsLessThan1024={false}
          viewPortIsLessThan1210={false}
        >
          <StackingContextProvider
            isTopBarHeaderStacked={false}
            isProgramHeaderStacked={false}
            isSearchResultsHeaderStacked={false}
            hasPreventScrollEnabled={false}
            preventScrollLayersCount={0}
          >
            <RequestContextProvider
              isFrontPage={false}
              realEstateContext={RealEstateContext.Unknown}
              location={null}
            >
              <PageContent
                menuMain={menuMain()!}
                menuFooter={menuFooter()!}
                settings={settings()!}
              >
                {props.children}
              </PageContent>
            </RequestContextProvider>
          </StackingContextProvider>
        </ViewportSizeContextProvider>
      </DrupalSettingsProvider>
    </>
  );
}
