import type { MenuItem, Settings } from "~/types/common";
import { useViewportSizeContext } from "~/contexts/ViewportSizeContext";
import { useStackingContext } from "~/contexts/StackingContext";
import { useSearchContext } from "~/contexts/SearchContext";
import { createEffect, createResource, createSignal } from "solid-js";
import { useMenuMobileContext } from "~/contexts/MenuMobileContext";

import { createPreventScroll } from "@solid-aria/overlays";
import Menu from "~/components/shared/Menu";
import { getMenuMain } from "~/lib/fetcher";

type TopBarProps = {
  menuMainItems: MenuItem[];
  settings: Settings;
};

export default function TopBar(props: TopBarProps) {
  const [viewportSizeProvider] = useViewportSizeContext();
  const [stackingContext] = useStackingContext();
  //const [searchState, { searchBarOpen }] = useSearchContext();
  //const [menuMobileContext, { popStackLevel }] = useMenuMobileContext();
  //const [, { sendClick }] = useEventsContext();
  const [, { incrementPreventScrollLayerCount }] = useStackingContext();

  const [drawerIsOpen, setDrawerIsOpen] = createSignal(false);

  function hideDrawerFromOverlay(event: MouseEvent) {
    if ((event.target as HTMLElement).classList.contains("nav-overlay")) {
      setDrawerIsOpen(false);
    }
  }

  createEffect(() => {
    if (drawerIsOpen() && viewportSizeProvider.viewPortIsLessThan1210) {
      createPreventScroll();
      incrementPreventScrollLayerCount!();
    }
  });

  return (
    <>
      <header
        role="banner"
        data-ga-zone="topbar"
        class="header"
        classList={{ stacked: stackingContext.isTopBarHeaderStacked }}
      >
        <h1>Topbar</h1>
        <button
          type="button"
          class="btn btn-icon show-menu-main"
          aria-label="Menu"
          onClick={() => setDrawerIsOpen(true)}
        >
          Drawer open
        </button>

        <nav role="navigation" id="menu-main">
          {/* class={`${menuMobileContext.stackLevelClass}`} */}
          <Menu name="main" items={props.menuMainItems} class="menu-main" />
        </nav>
      </header>
    </>
  );
}
