import type { ParentComponent } from "solid-js";
import { createContext, useContext } from "solid-js";
import { createStore } from "solid-js/store";
import type { Settings } from "~/types/common";

export type DrupalSettingsContextState = {
  settings?: Settings;
};

export type DrupalSettingsContextStore = [
  DrupalSettingsContextState,
  { setSettings?: (settings: Settings) => void },
];

const DrupalSettingsContext = createContext<DrupalSettingsContextStore>([
  {
    settings: undefined,
  },
  {},
]);

export const DrupalSettingsProvider: ParentComponent<{
  settings: Settings;
}> = (props) => {
  const [state, setState] = createStore({
      settings: props.settings || undefined,
    }),
    store: DrupalSettingsContextStore = [
      state,
      {
        setSettings(settings: Settings) {
          setState("settings", settings);
        },
      },
    ];

  return (
    <DrupalSettingsContext.Provider value={store}>
      {props.children}
    </DrupalSettingsContext.Provider>
  );
};

export function useDrupalSettingsContext() {
  return useContext(DrupalSettingsContext);
}
