import type { ParentComponent } from "solid-js";
import { createContext, useContext, DEV } from "solid-js";
import { createStore } from "solid-js/store";
import type { MediaOverlayType } from "~/components/Shared/MediaOverlay";

export type MediaOverlayContext = [
  { type?: MediaOverlayType; values?: unknown },
  {
    setMediaOverlay: (overlay_type: string, overlay_values: unknown) => void;
    unsetMediaOverlay: () => void;
  },
];

const MediaOverlayContext = createContext<MediaOverlayContext>([
  { type: null, values: null },
  {
    setMediaOverlay: () => undefined,
    unsetMediaOverlay: () => undefined,
  },
]);

export const MediaOverlayProvider: ParentComponent<{
  type?: MediaOverlayType;
  values?: unknown;
}> = (props) => {
  const [state, setState] = createStore({
      // eslint-disable-next-line solid/reactivity
      type: props.type || null,
      // eslint-disable-next-line solid/reactivity
      values: props.values || null,
    }),
    store: MediaOverlayContext = [
      state,
      {
        setMediaOverlay(
          overlay_type: MediaOverlayType,
          overlay_values: unknown,
        ) {
          setState("type", overlay_type);
          setState("values", overlay_values);
          if (DEV) {
            console.groupCollapsed("🎬 MediaOverlay opening");
            console.log("Type:", state.type);
            console.log("Values:", state.values);
            console.groupEnd();
          }
        },
        unsetMediaOverlay() {
          setState("type", null);
          setState("values", null);
        },
      },
    ];

  return (
    <MediaOverlayContext.Provider value={store}>
      {props.children}
    </MediaOverlayContext.Provider>
  );
};

export function useMediaOverlayContext() {
  return useContext(MediaOverlayContext);
}
