import type { ParentComponent } from "solid-js";
import { createContext, useContext } from "solid-js";
import { createStore } from "solid-js/store";
import type { Location } from "@solidjs/router";
import { RealEstateContext } from "~/types/common";

type RequestContextStore = [
  {
    isFrontPage: boolean;
    hasGmapsLoaded: boolean;
    hasGlideRequested: boolean;
    hasGlideLoaded: boolean;
    hasMapboxLoaded: boolean;
    realEstateContext: RealEstateContext;
    location: Location | null;
  },
  {
    setIsFrontPage?: (isFront: boolean) => void;
    setRealEstateContext?: (context: RealEstateContext) => void;
    setGlideRequested?: (hasLoaded: boolean) => void;
    setHasGlideLoaded?: (hasLoaded: boolean) => void;
    setHasGmapsLoaded?: (hasLoaded: boolean) => void;
    setHasMapboxLoaded?: (hasLoaded: boolean) => void;
  },
];

const RequestContext = createContext([
  {
    isFrontPage: false,
    realEstateContext: RealEstateContext.Unknown,
    hasGmapsLoaded: false,
    hasGlideRequested: false,
    hasGlideLoaded: false,
    hasMapboxLoaded: false,
    location: null,
  },
  {},
]);

export const RequestContextProvider: ParentComponent<{
  isFrontPage: boolean;
  realEstateContext: RealEstateContext;
  location: Location | null;
}> = (props) => {
  const [state, setState] = createStore({
      isFrontPage: props.isFrontPage || false,
      realEstateContext: props.realEstateContext || RealEstateContext.Unknown,
      hasGmapsLoaded: false,
      hasGlideRequested: false,
      hasGlideLoaded: false,
      hasMapboxLoaded: false,
      location: props.location || undefined,
    }),
    store: RequestContextStore = [
      state,
      {
        setIsFrontPage(isFront: boolean) {
          setState("isFrontPage", isFront);
        },
        setRealEstateContext(context: RealEstateContext) {
          setState("realEstateContext", context);
        },
        setHasGmapsLoaded(loaded: boolean) {
          setState("hasGmapsLoaded", loaded);
        },
        setGlideRequested(loaded: boolean) {
          setState("hasGlideRequested", loaded);
        },
        setHasGlideLoaded(loaded: boolean) {
          setState("hasGlideLoaded", loaded);
        },
        setHasMapboxLoaded(loaded: boolean) {
          setState("hasMapboxLoaded", loaded);
        },
      },
    ];

  return (
    <RequestContext.Provider value={store}>
      {props.children}
    </RequestContext.Provider>
  );
};

export function useRequestContext() {
  return useContext(RequestContext);
}
