import { createContext, useContext, type ParentComponent } from "solid-js";
import { createStore } from "solid-js/store";

export type SearchResultsCountResponse = {
  count: number;
  count_preview: number;
  count_proximity: number;
};

export type SearchContext = [
  {
    searchBarIsOpen: boolean;
    advancedSearchIsOpen: boolean;
    searchResultsCountResponse: SearchResultsCountResponse;
  },
  {
    searchBarOpen?: () => void;
    searchBarClose?: () => void;
    advancedSearchOpen?: () => void;
    advancedSearchClose?: () => void;
    setSearchResultsCountResponse?: (
      results: SearchResultsCountResponse,
    ) => void;
  },
];

const SearchContext = createContext<SearchContext>([
  {
    searchBarIsOpen: false,
    advancedSearchIsOpen: false,
    searchResultsCountResponse: {
      count: 0,
      count_preview: 0,
      count_proximity: 0,
    },
  },
  {},
]);

export const SearchProvider: ParentComponent = (props) => {
  const [searchState, setState] = createStore({
      searchBarIsOpen: false,
      advancedSearchIsOpen: false,
      searchResultsCountResponse: {
        count: 0,
        count_preview: 0,
        count_proximity: 0,
      },
    }),
    store: SearchContext = [
      searchState,
      {
        searchBarOpen() {
          setState("searchBarIsOpen", true);
        },
        searchBarClose() {
          setState("searchBarIsOpen", false);
        },
        advancedSearchOpen() {
          setState("advancedSearchIsOpen", true);
        },
        advancedSearchClose() {
          setState("advancedSearchIsOpen", false);
        },
        setSearchResultsCountResponse(results: SearchResultsCountResponse) {
          setState("searchResultsCountResponse", results);
        },
      },
    ];

  return (
    <SearchContext.Provider value={store}>
      {props.children}
    </SearchContext.Provider>
  );
};

export function useSearchContext() {
  return useContext(SearchContext);
}
