import TaggedRedisCache from "~/lib/TaggedRedisCache";

const redisOptions = { defaultTimeout: 86400, redis: { keyPrefix: "solid:" } };

const REDIS_PORT = import.meta.env.VITE_KB_REDIS_PORT ?? 6379;
const REDIS_HOST = import.meta.env.VITE_KB_REDIS_HOST ?? "localhost";

export async function CacheGet(path: string): Promise<unknown> {
  const cache = new TaggedRedisCache(REDIS_PORT, REDIS_HOST, redisOptions);
  return cache.get(path);
}

export async function CacheSet(
  key: string,
  data: unknown,
  tags: string[],
  expire: number,
) {
  const cache = new TaggedRedisCache(REDIS_PORT, REDIS_HOST, redisOptions);

  let options = {};

  if (expire && expire > 0) {
    options = { timeout: expire };
  }

  cache
    .set(key, data, tags, options)
    .then((success) => {
      if (import.meta.env.VITE_KB_CACHE_DEBUG == 1) {
        console.log("CacheSet success", success);
      }
    })
    .catch((err) => {
      if (import.meta.env.VITE_KB_CACHE_DEBUG == 1) {
        console.log("CacheSet error", err);
      }
    });
}
