import type { Prettify } from "~/types/common";

export type ContentEntity = {
  type: string;
  id: string; // uuid
  drupal_internal__nid: number;
  drupal_internal__vid: number;
  langcode: string;
  revision_timestamp: string;
  status: boolean;
  title: string;
  created: string;
  changed: string;
  promote: boolean;
  sticky: boolean;
  default_langcode: boolean;
  revision_translation_affected: boolean;
  metatag: Metatag[];
  path: Path;
  publish_on?: any; // @todo type
  unpublish_on?: any; // @todo type
  published_at: string;
  node_type: {
    type: string;
    id: string;
    meta: {
      drupal_internal__target_id: string;
    };
  };
  revision_uid: {
    type: string;
    id: string;
    meta: {
      drupal_internal__target_id: number | string;
    };
  };
  uid: {
    type: string;
    id: string;
    meta: {
      drupal_internal__target_id: number | string;
    };
  };
};

export type Program = Prettify<
  ContentEntity & {
    field_additional: string;
    field_address_hide: boolean;
    field_address_override?: string;
    field_administrator?: string;
    field_archived: boolean;
    field_basements_count: 0;
    field_bbc: boolean;
    field_building_text: TextWithFormatField;
    field_catchline?: string;
    field_cellars_count: number;
    field_cities_nearby: string[];
    field_city: string;
    field_classified?: string;
    field_date_delivery_hide: boolean;
    field_date_display_prices?: string;
    field_disp_grid_bare_ownership: boolean;
    field_disp_grid_lmnp: boolean;
    field_disp_grid_patrimonial: boolean;
    field_disp_grid_pinel: boolean;
    field_disp_grid_pinel_plus: boolean;
    field_disp_grid_vat_inc: boolean;
    field_disp_grid_vat_inc_brs: boolean;
    field_disp_grid_vat_inc_mastered: boolean;
    field_disp_grid_vat_inc_reduced: boolean;
    field_district_text: TextWithFormatField;
    field_exported: boolean;
    field_freehold_percentage?: string;
    field_gps: GPSField;
    field_had_lots_previously: boolean;
    field_hqe: boolean;
    field_images_export: boolean;
    field_invest_text: TextWithFormatField;
    field_invest_title?: string;
    field_is_cogehome3d: boolean;
    field_key_figures_text?: string;
    field_lease_term?: string;
    field_lease_term_lmnp?: string;
    field_legals?: string;
    field_lots_count: number;
    field_lots_txt: TextWithFormatField;
    field_main: boolean;
    field_main_text: TextWithFormatField;
    field_manager_text?: string;
    field_managers?: string;
    field_meta?: string;
    field_monthly_payment_amount?: string;
    field_monthly_payments_text: TextWithFormatField;
    field_monthly_payments_title?: string;
    field_nf: boolean;
    field_nf_habitat: boolean;
    field_nf_habitat_hqe: boolean;
    field_oos: boolean;
    field_parkings_count: number;
    field_postal_code?: string;
    field_program_id?: string;
    field_program_ids?: string;
    field_re_2020: boolean;
    field_re_2020_2025: boolean;
    field_rt_2012: boolean;
    field_scale: number;
    field_sea_desc?: string;
    field_services_disp: boolean;
    field_services_txt?: string;
    field_so_additional?: string;
    field_so_block_closed: boolean;
    field_so_block_std: boolean;
    field_so_block_visio: boolean;
    field_so_city?: string;
    field_so_gps?: GPSField;
    field_so_opening?: string;
    field_so_phone?: string;
    field_so_postal_code?: string;
    field_so_scale: number;
    field_so_street?: string;
    field_street?: string;
    field_strenghts?: string;
    field_video_first_screen: boolean;

    node_type: RelationshipField;
    revision_uid: RelationshipField;
    uid: RelationshipField;

    field_3d?: Field3D;
    field_3d_provider?: TermEntityReferenceField & {
      field_3dr_url: string;
      title?: string;
    };

    field_agency: any;
    field_building_img?: ImageField;
    field_commercial_specificities: TermEntityReferenceField[];
    field_district_img?: ImageField;
    field_division: any;
    field_dwelling_type?: TermEntityReferenceField;

    field_features: TermEntityReferenceField &
      {
        field_icon: ImageField;
      }[];
    field_form: any;
    field_geography: any;

    field_image_desktop?: ImageField[];
    field_image_mobile?: ImageField;
    field_leaflet?: FileField;
    field_lots_img?: ImageField;
    field_manager_img?: ImageField;
    field_product_type_default?: TermEntityReferenceField;
    field_program_type?: TermEntityReferenceField;
    field_push_page: any;
    field_push_top: any;
    field_services: any;

    field_sales_state?: TermEntityReferenceField;

    field_strengths: any;

    field_video: any;
    field_tours: any;
  }
>;

export type TermEntity = {
  tid: number;
  metatags?: Metatag[];
  uuid: string;
  revision_id: number;
  langcode: string;
  vid: {
    target_id: string;
  };
  status: boolean;
  name: string;
  description: TextWithFormatField;
  changed: number;
  default_langcode: boolean;
  revision_default: boolean;
  path: Path;
};

export type EntityReferenceField = {
  target_id: number | string;
};

export type Path = {
  alias: string;
  pid: number;
  langcode: string;
};

export type Metatag = {
  tag: string;
  attributes: Record<string, string>;
};

export type TextWithFormatField = {
  value: string;
  format: "full_html" | "plain_text" | "email_html" | "html_code";
  processed: string;
};

export type TermEntityReferenceField = {
  drupal_internal__tid: number;
  name: string;
  type: string;
  alias: string;
  description?: TextWithFormatField;
  status?: boolean;
  metatags?: Metatag[];
  path?: Path;
  weight?: number;
};

export type RelationshipField = {
  type: string;
  id: string;
  meta: {
    drupal_internal__target_id: number;
  };
};

export type GPSField = {
  value: string;
  geo_type: string;
  lat: number;
  lon: number;
  left: number;
  top: number;
  right: number;
  bottom: number;
  geohash: string;
  latlon: string;
};

export type FileField = {
  id: string;
  drupal_internal__fid: number;
  langcode: string;
  filename?: string;
  uri: {
    value: string;
    url: string;
  };
  filemime?: string;
  filesize?: number;
  created?: string;
  changed?: string;
  meta?: {
    display: boolean;
    drupal_internal__target_id?: number;
  };
};

export type ImageField = {
  id: string;
  drupal_internal__fid: number;
  langcode: string;
  filename?: string;
  uri: {
    value: string;
    url: string;
  };
  filemime?: string;
  filesize?: number;
  created?: string;
  changed?: string;
  meta?: {
    alt?: string;
    title?: string;
    width?: number;
    height?: number;
    drupal_internal__target_id?: number;
  };
};

export type Field3D = {
  type: "paragraph--3d";
  status: boolean;
  created: string;
  field_3d_url: {
    uri: string;
    title: string;
    options: any[];
  };
};
