import type { RsPathType, Settings } from "~/types/common";
import { useDrupalSettingsContext } from "~/contexts/DrupalSettingsContext";

/**
 * Returns the path part of a URL.
 *
 * - Input:  /de/jobs/berlin?foo=bar#baz
 * - Output: /de/jobs/berlin
 *
 * @param url
 */
export function getPathFromUrl(url: string): string {
  return url.split(/[?#]/)[0];
}

/**
 * Given a URL, returns it with the query parameter r=s if the config for that path type it enabled.
 */
export function urlRs(
  type: RsPathType,
  url: string,
  settings?: Settings,
): string {
  let enabled;
  if (settings) {
    enabled = settings.solid_config[`${type}_enabled`];
  } else {
    const [settingsContext] = useDrupalSettingsContext();
    enabled = settingsContext.settings?.solid_config[`${type}_enabled`];
  }

  if (enabled && url.indexOf("r=s") === -1) {
    return url + (url.indexOf("?") > -1 ? "&" : "?") + "r=s";
  }
  return url;
}
